@@ -9,13 +9,13 @@ from numpy import argsort, array, concatenate, nonzero, invert, take, \
 
 # Enthought library imports
 from kiva.constants import STROKE
-from traits.api import Dict, Enum, Float, Instance
+from traits.api import Dict, Enum, Float, Instance, on_trait_change
 from traitsui.api import Item, RangeEditor
 
 # Local, relative imports
 from array_data_source import ArrayDataSource
 from base import left_shift, right_shift
-from color_mapper import ColorMapper
+from abstract_colormap import AbstractColormap
 from scatterplot import ScatterPlot, ScatterPlotView
 
 
@@ -43,7 +43,7 @@ class ColormappedScatterPlot(ScatterPlot):
     color_data = Instance(ArrayDataSource)
 
     # Mapping for colors.
-    color_mapper = Instance(ColorMapper)
+    color_mapper = Instance(AbstractColormap)
 
     # The alpha value to apply to the result of the color-mapping process.
     # (This makes it easier to create color maps without having to worry
@@ -414,6 +414,11 @@ class ColormappedScatterPlot(ScatterPlot):
 
         return
 
+    @on_trait_change('color_mapper:updated')
+    def _color_mapper_updated(self):
+        self.invalidate_draw()
+        self.request_redraw()
+
     def _fill_alpha_changed(self):
         self.invalidate_draw()
         self.request_redraw()
