@@ -1,6 +1,7 @@
 """ Defines the DataLabel class and related trait and function.
 """
 # Major library imports
+from math import sqrt
 from numpy import array, asarray, inf
 from numpy.linalg import norm
 
@@ -15,11 +16,13 @@ from tooltip import ToolTip
 
 
 # Specifies the position of a label relative to its target.  This can
-# be one of the text strings indicated, or a tuple or list of floats representing
-# the (x_offset, y_offset) in screen space of the label's lower left corner.
+# be one of the text strings indicated, or a tuple or list of floats
+# representing the (x_offset, y_offset) in screen space of the label's
+# lower left corner.
 LabelPositionTrait = Trait("top right",
                            Enum("bottom", "left", "right", "top",
-                                "top right", "top left", "bottom left", "bottom right"),
+                                "top right", "top left",
+                                "bottom left", "bottom right"),
                            Tuple, List)
 
 
@@ -66,24 +69,24 @@ def draw_arrow(gc, pt1, pt2, color, arrowhead_size=10.0, offset1=0,
         pt1 = asarray(pt1)
         pt2 = asarray(pt2)
 
-        unit_vec = (pt2-pt1)
+        unit_vec = pt2 - pt1
         unit_vec /= norm(unit_vec)
 
         if unit_vec[0] == 0:
-            perp_vec = array((0.3 * arrowhead_size,0))
+            perp_vec = array((0.3 * arrowhead_size, 0))
         elif unit_vec[1] == 0:
-            perp_vec = array((0,0.3 * arrowhead_size))
+            perp_vec = array((0, 0.3 * arrowhead_size))
         else:
-            slope = unit_vec[1]/unit_vec[0]
-            perp_slope = -1/slope
+            slope = unit_vec[1] / unit_vec[0]
+            perp_slope = -1 / slope
             perp_vec = array((1.0, perp_slope))
             perp_vec *= 0.3 * arrowhead_size / norm(perp_vec)
 
         pt1 = pt1 + offset1 * unit_vec
         pt2 = pt2 - offset2 * unit_vec
 
-        arrowhead_l = pt2 - (arrowhead_size*unit_vec + perp_vec)
-        arrowhead_r = pt2 - (arrowhead_size*unit_vec - perp_vec)
+        arrowhead_l = pt2 - (arrowhead_size * unit_vec + perp_vec)
+        arrowhead_r = pt2 - (arrowhead_size * unit_vec - perp_vec)
         arrow = (pt1, pt2, arrowhead_l, arrowhead_r)
     else:
         pt1, pt2, arrowhead_l, arrowhead_r = arrow
@@ -107,8 +110,52 @@ def draw_arrow(gc, pt1, pt2, color, arrowhead_size=10.0, offset1=0,
     return arrow
 
 
+def find_region(px, py, x, y, x2, y2):
+    """Classify the location of the point (px, py) relative to a rectangle.
+
+    (x, y) and (x2, y2) are the lower-left and upper-right corners of the
+    rectangle, respectively.  (px, py) is classified as "left", "right",
+    "top", "bottom" or "inside", according to the following diagram:
+
+            \     top      /
+             \            /
+              +----------+
+         left |  inside  | right
+              +----------+
+             /            \ 
+            /    bottom    \ 
+
+    """
+    if px < x:
+        dx = x - px
+        if py > y2 + dx:
+            region = 'top'
+        elif py < y - dx:
+            region = 'bottom'
+        else:
+            region = 'left'
+    elif px > x2:
+        dx = px - x2
+        if py > y2 + dx:
+            region = 'top'
+        elif py < y - dx:
+            region = 'bottom'
+        else:
+            region = 'right'
+    else:  # x <= px <= x2
+        if py > y2:
+            region = 'top'
+        elif py < y:
+            region = 'bottom'
+        else:
+            region = 'inside'
+    return region
+
+
 class DataLabel(ToolTip):
-    """ A label on a point in data space, optionally with an arrow to the point.
+    """ A label on a point in data space.
+
+    Optionally, an arrow is drawn to the point.
     """
 
     # The symbol to use if **marker** is set to "custom". This attribute must
@@ -139,10 +186,18 @@ class DataLabel(ToolTip):
 
     # The center x position (average of x and x2)
     xmid = Property(Float, depends_on=['x', 'x2'])
-    
+
     # The center y position (average of y and y2)
     ymid = Property(Float, depends_on=['y', 'y2'])
 
+    # 'box' is a simple rectangular box, with an arrow that is a single line
+    # with an arrowhead at the data point.
+    # 'bubble' can be given rounded corners (by setting `corner_radius`), and
+    # the 'arrow' is a thin triangular wedge with its point at the data point.
+    # When label_style is 'bubble', the following traits are ignored:
+    #    arrow_size, arrow_color, arrow_root, and arrow_max_length.
+    label_style = Enum('box', 'bubble')
+
     #----------------------------------------------------------------------
     # Marker traits
     #----------------------------------------------------------------------
@@ -154,11 +209,12 @@ class DataLabel(ToolTip):
     # keys.
     marker = MarkerTrait
 
-    # The pixel size of the marker (doesn't include the thickness of the outline).
+    # The pixel size of the marker (doesn't include the thickness of the
+    # outline).
     marker_size = Int(4)
 
-    # The thickness, in pixels, of the outline to draw around the marker.  If
-    # this is 0, no outline will be drawn.
+    # The thickness, in pixels, of the outline to draw around the marker.
+    # If this is 0, no outline will be drawn.
     marker_line_width = Float(1.0)
 
     # The color of the inside of the marker.
@@ -182,8 +238,9 @@ class DataLabel(ToolTip):
     arrow_color = ColorTrait("black")
 
     # The position of the base of the arrow on the label.  If this
-    # is 'auto', then the label uses **label_position**.  Otherwise, it treats
-    # the label as if it were at the label position indicated by this attribute.
+    # is 'auto', then the label uses **label_position**.  Otherwise, it
+    # treats the label as if it were at the label position indicated by
+    # this attribute.
     arrow_root = Trait("auto", "auto", "top left", "top right", "bottom left",
                        "bottom right", "top center", "bottom center",
                        "left center", "right center")
@@ -196,6 +253,13 @@ class DataLabel(ToolTip):
     # the arrow will be drawn regardless of how long it is.
     arrow_max_length = Float(inf)
 
+    #----------------------------------------------------------------------
+    # Bubble traits
+    #----------------------------------------------------------------------
+
+    # The radius (in screen coordinates) of the curved corners of the "bubble".
+    corner_radius = Float(10)
+
     #-------------------------------------------------------------------------
     # Private traits
     #-------------------------------------------------------------------------
@@ -205,9 +269,9 @@ class DataLabel(ToolTip):
 
     _cached_arrow = Any
 
-    # When **arrow_root** is 'auto', this determines the location on the data label
-    # from which the arrow is drawn, based on the position of the label relative
-    # to its data point.
+    # When **arrow_root** is 'auto', this determines the location on the data
+    # label from which the arrow is drawn, based on the position of the label
+    # relative to its data point.
     _position_root_map = {
         "top left": "bottom right",
         "top right": "bottom left",
@@ -230,7 +294,6 @@ class DataLabel(ToolTip):
         "right center": ("x2", "ymid"),
         }
 
-
     def overlay(self, component, gc, view_bounds=None, mode="normal"):
         """ Draws the tooltip overlaid on another component.
 
@@ -243,6 +306,24 @@ class DataLabel(ToolTip):
 
         self.do_layout()
 
+        if self.label_style == 'box':
+            self._render_box(component, gc, view_bounds=view_bounds,
+                             mode=mode)
+        else:
+            self._render_bubble(component, gc, view_bounds=view_bounds,
+                                mode=mode)
+
+        # draw the marker
+        if self.marker_visible:
+            render_markers(gc, [self._screen_coords],
+                           self.marker, self.marker_size,
+                           self.marker_color_, self.marker_line_width,
+                           self.marker_line_color_, self.custom_symbol)
+
+        if self.clip_to_plot:
+            gc.restore_state()
+
+    def _render_box(self, component, gc, view_bounds=None, mode='normal'):
         # draw the arrow if necessary
         if self.arrow_visible:
             if self._cached_arrow is None:
@@ -253,21 +334,22 @@ class DataLabel(ToolTip):
                         arrow_root = self.label_position
                     else:
                         arrow_root = self.arrow_root
-                    ox, oy = self._root_positions.get(
-                                 self._position_root_map.get(arrow_root, "DUMMY"),
-                                 (self.x+self.width/2, self.y+self.height/2)
-                                 )
+                    pos = self._position_root_map.get(arrow_root, "DUMMY")
+                    ox, oy = self._root_positions.get(pos,
+                                        (self.x + self.width / 2,
+                                         self.y + self.height / 2))
 
                 if type(ox) == str:
                     ox = getattr(self, ox)
                     oy = getattr(self, oy)
-                self._cached_arrow = draw_arrow(gc, (ox, oy), self._screen_coords,
-                                                self.arrow_color_,
-                                                arrowhead_size=self.arrow_size,
-                                                offset1=3,
-                                                offset2=self.marker_size+3,
-                                                minlen=self.arrow_min_length,
-                                                maxlen=self.arrow_max_length)
+                self._cached_arrow = draw_arrow(gc, (ox, oy),
+                                            self._screen_coords,
+                                            self.arrow_color_,
+                                            arrowhead_size=self.arrow_size,
+                                            offset1=3,
+                                            offset2=self.marker_size + 3,
+                                            minlen=self.arrow_min_length,
+                                            maxlen=self.arrow_max_length)
             else:
                 draw_arrow(gc, None, None, self.arrow_color_,
                            arrow=self._cached_arrow,
@@ -277,14 +359,102 @@ class DataLabel(ToolTip):
         # layout and render the label itself
         ToolTip.overlay(self, component, gc, view_bounds, mode)
 
-        # draw the marker
-        if self.marker_visible:
-            render_markers(gc, [self._screen_coords], self.marker, self.marker_size,
-                           self.marker_color_, self.marker_line_width,
-                           self.marker_line_color_, self.custom_symbol)
+    def _render_bubble(self, component, gc, view_bounds=None, mode='normal'):
+        """ Render the bubble label in the graphics context. """
+        # (px, py) is the data point in screen space.
+        px, py = self._screen_coords
 
-        if self.clip_to_plot:
-            gc.restore_state()
+        # (x, y) is the lower left corner of the label.
+        x = self.x
+        y = self.y
+        # (x2, y2) is the upper right corner of the label.
+        x2 = self.x2
+        y2 = self.y2
+        # r is the corner radius.
+        r = self.corner_radius
+
+        if self.arrow_visible:
+            # FIXME: Make 'gap_width' a configurable trait (and give it a
+            #        better name).
+            max_gap_width = 10
+            gap_width = min(max_gap_width,
+                            abs(x2 - x - 2 * r),
+                            abs(y2 - y - 2 * r))
+            region = find_region(px, py, x, y, x2, y2)
+
+            # Figure out where the "arrow" connects to the "bubble".
+            if region == 'left' or region == 'right':
+                gap_start = py - gap_width / 2
+                if gap_start < y + r:
+                    gap_start = y + r
+                elif gap_start > y2 - r - gap_width:
+                    gap_start = y2 - r - gap_width
+                by = gap_start + 0.5 * gap_width
+                if region == 'left':
+                    bx = x
+                else:
+                    bx = x2
+            else:
+                gap_start = px - gap_width / 2
+                if gap_start < x + r:
+                    gap_start = x + r
+                elif gap_start > x2 - r - gap_width:
+                    gap_start = x2 - r - gap_width
+                bx = gap_start + 0.5 * gap_width
+                if region == 'top':
+                    by = y2
+                else:
+                    by = y
+
+        arrow_len = sqrt((px - bx) ** 2 + (py - by) ** 2)
+        arrow_visible = (self.arrow_visible and
+                         (arrow_len >= self.arrow_min_length))
+
+        with gc:
+            if self.border_visible:
+                gc.set_line_width(self.border_width)
+                gc.set_stroke_color(self.border_color_)
+            else:
+                gc.set_line_width(0)
+                gc.set_stroke_color((0, 0, 0, 0))
+            gc.set_fill_color(self.bgcolor_)
+
+            # Start at the lower left, on the left edge where the curved
+            # part of the box ends.
+            gc.move_to(x, y + r)
+
+            # Draw the left side and the upper left curved corner.
+            if arrow_visible and region == 'left':
+                gc.line_to(x, gap_start)
+                gc.line_to(px, py)
+                gc.line_to(x, gap_start + gap_width)
+            gc.arc_to(x, y2, x + r, y2, r)
+
+            # Draw the top and the upper right curved corner.
+            if arrow_visible and region == 'top':
+                gc.line_to(gap_start, y2)
+                gc.line_to(px, py)
+                gc.line_to(gap_start + gap_width, y2)
+            gc.arc_to(x2, y2, x2, y2 - r, r)
+
+            # Draw the right side and the lower right curved corner.
+            if arrow_visible and region == 'right':
+                gc.line_to(x2, gap_start + gap_width)
+                gc.line_to(px, py)
+                gc.line_to(x2, gap_start)
+            gc.arc_to(x2, y, x2 - r, y, r)
+
+            # Draw the bottom and the lower left curved corner.
+            if arrow_visible and region == 'bottom':
+                gc.line_to(gap_start + gap_width, y)
+                gc.line_to(px, py)
+                gc.line_to(gap_start, y)
+            gc.arc_to(x, y, x, y + r, r)
+
+            # Finish the "bubble".
+            gc.draw_path()
+
+            self._draw_overlay(gc)
 
     def _do_layout(self, size=None):
         """Computes the size and position of the label and arrow.
@@ -318,11 +488,11 @@ class DataLabel(ToolTip):
                     self.outer_y = sy
             if "center" in orientation:
                 if " " not in orientation:
-                    self.x = sx - (self.width/2)
-                    self.y = sy - (self.height/2)
+                    self.x = sx - (self.width / 2)
+                    self.y = sy - (self.height / 2)
                 else:
-                    self.x = sx - (self.outer_width/2) - 1
-                    self.y = sy - (self.outer_height/2) - 1
+                    self.x = sx - (self.outer_width / 2) - 1
+                    self.y = sy - (self.outer_height / 2) - 1
         else:
             self.x = sx + self.label_position[0]
             self.y = sy + self.label_position[1]
@@ -347,7 +517,8 @@ class DataLabel(ToolTip):
         pt = self.data_point
         if pt is not None:
             if self.show_label_coords:
-                self.lines = [self.label_text, self.label_format % {"x": pt[0], "y": pt[1]}]
+                self.lines = [self.label_text,
+                              self.label_format % {"x": pt[0], "y": pt[1]}]
             else:
                 self.lines = [self.label_text]
 
@@ -355,32 +526,37 @@ class DataLabel(ToolTip):
         for comp, attach in ((old, False), (new, True)):
             if comp is not None:
                 if hasattr(comp, 'index_mapper'):
-                    self._modify_mapper_listeners(comp.index_mapper, attach=attach)
+                    self._modify_mapper_listeners(comp.index_mapper,
+                                                  attach=attach)
                 if hasattr(comp, 'value_mapper'):
-                    self._modify_mapper_listeners(comp.value_mapper, attach=attach)
+                    self._modify_mapper_listeners(comp.value_mapper,
+                                                  attach=attach)
         return
 
     def _modify_mapper_listeners(self, mapper, attach=True):
         if mapper is not None:
-            mapper.on_trait_change(self._handle_mapper, 'updated', remove=not attach)
+            mapper.on_trait_change(self._handle_mapper, 'updated',
+                                   remove=not attach)
         return
 
     def _handle_mapper(self):
-        # This gets fired whenever a mapper on our plot fires its 'updated' event.
+        # This gets fired whenever a mapper on our plot fires its
+        # 'updated' event.
         self._layout_needed = True
 
-    @on_trait_change("arrow_size,arrow_root,arrow_min_length,arrow_max_length")
+    @on_trait_change("arrow_size,arrow_root,arrow_min_length," +
+                     "arrow_max_length")
     def _invalidate_arrow(self):
         self._cached_arrow = None
         self._layout_needed = True
 
-    @on_trait_change("label_position,position,position_items,bounds,bounds_items")
+    @on_trait_change("label_position,position,position_items,bounds," +
+                     "bounds_items")
     def _invalidate_layout(self):
         self._layout_needed = True
 
-
     def _get_xmid(self):
         return 0.5 * (self.x + self.x2)
-    
+
     def _get_ymid(self):
         return 0.5 * (self.y + self.y2)
