@@ -2,4 +2,5 @@ from databox import DataBox
 from container_overlay import ContainerOverlay
 from aligned_container_overlay import AlignedContainerOverlay
 from text_grid_overlay import TextGridOverlay
-from simple_inspector_overlay import SimpleInspectorOverlay
+from simple_inspector_overlay import SimpleInspectorOverlay, basic_formatter, \
+    datetime_formatter, date_formatter, time_formatter
