@@ -157,7 +157,7 @@ marker_trans = {
     '+': 'plus',
     's': 'square',
     '^': 'triangle',
-    'v': 'down triangle'
+    'v': 'inverted_triangle'
 }
 
 line_re = re.compile('--|-\.|[-:]')
@@ -285,9 +285,7 @@ def do_plot(plotdata, active_plot, *data_and_formats, **kwtraits):
             format["marker"] = marker
             format["color"] = markercolor
 
-        # Check the data sort order, but only if it will create a new datasource
-        if x not in active_plot.datasources:
-            x_sort_order = _check_sort_order(plotdata.get_data(x))
+        x_sort_order = _check_sort_order(plotdata.get_data(x))
         plots.extend(active_plot.plot((x,y), type=",".join(plot_type), **format))
 
         # Set the sort order
@@ -327,7 +325,7 @@ def do_imshow(plotdata, active_plot, *data, **kwargs):
     x = None
     y = None
     try:
-        z = _get_or_create_plot_data(data[0])
+        z = _get_or_create_plot_data(data[0], plotdata)
     except ValueError:
         # z is the name of the file
         # create plot data
@@ -348,7 +346,7 @@ def do_pcolor(plotdata, colormap, active_plot, *data, **kwargs ):
     if len(data) == 1:
         x = None
         y = None
-        z = _get_or_create_plot_data(data[0])
+        z = _get_or_create_plot_data(data[0], plotdata)
 
     # three data sources means we got x-y grid data of some sort, too
     elif len(data) == 3:
@@ -372,7 +370,7 @@ def do_contour(plotdata, colormap, active_plot, type, *data, **kwargs ):
     if len(data) == 1:
         x = None
         y = None
-        z = _get_or_create_plot_data(data[0])
+        z = _get_or_create_plot_data(data[0], plotdata)
 
     # three data sources means we got x-y grid data of some sort, too
     elif len(data) == 3:
