@@ -125,7 +125,7 @@ if ETSConfig.toolkit == "wx":
 
 elif ETSConfig.toolkit == "qt4":
 
-    from pyface.qt import QtGui
+    from pyface.qt import QtCore, QtGui
 
     class PlotWindow(QtGui.QFrame):
         """ A window for holding top-level plot containers.
@@ -137,6 +137,9 @@ elif ETSConfig.toolkit == "qt4":
         def __init__(self, is_image=False, bgcolor="white",
                      image_default_origin="top left", *args, **kw):
 
+            if 'size' in kw and isinstance(kw['size'], tuple):
+                # Convert to a QSize.
+                kw['size'] = QtCore.QSize(*kw['size'])
             super(PlotWindow, self).__init__(None, *args, **kw )
 
             # Some defaults which should be overridden by preferences.
@@ -163,8 +166,8 @@ elif ETSConfig.toolkit == "qt4":
             layout.addWidget(self.plot_window.control)
             self.setLayout(layout)
 
-            size = kw.setdefault("size", (600,600))
-            self.set_size(*size)
+            size = kw.get("size", QtCore.QSize(600,600))
+            self.set_size(size.width(), size.height())
 
             self.show()
 
