@@ -1,6 +1,17 @@
 
+# This contains python implementations of all the speedups
 from _speedups_fallback import *
 
+
+# cython implementation of speedups. Import these if we can.
+try:
+    from _cython_speedups import *
+except ImportError:
+    pass
+
+
+
+
 #try:
 #    from numpy import asarray
 #    import _speedups
